@@ -1,17 +1,16 @@
-import re
-import inspect
-from matchers import *
-from matchers import _Matchers_
-
 """
 Authors: www.iExpertos.com
-   - Carlos Ble (www.carlosble.com) 
+   - Carlos Ble (www.carlosble.com)
 
 License: Apache 2 (http://www.apache.org/licenses/LICENSE-2.0.html)
 
 Project home: http://www.pydoubles.org
 """
 
+import inspect
+from matchers import _MatchFinder_
+import safeunicode
+
 ANY_ARG = "<___ANY___ARG___>"
 
 class UnexpectedBehavior(Exception):
@@ -26,31 +25,52 @@ class ArgsDontMatch(Exception):
 class ApiMismatch(Exception):
     pass
 
-
-class _MethodHandler_():
+class _MethodHandler_(object):
     """
     Internal framework class. When user access a method or
     any other property in the test double, the framework
     return an instance of this class, which is callable.
     So calling this instance with parenthesis triggers
     the on_method_call event in the test double.
-    """    
-    
+    """
+    def callable_action(self, args, kwargs):
+        pass
+        
+    def __call__(self, *args, **kwargs):
+        return self.callable_action(args, kwargs)
+
+class _DoubleMethodHandler_(_MethodHandler_):
     def __init__(self, double, attr):
         self.double = double
         self.attr_name = attr
-        
-    def __call__(self, *args, **kwargs):
+
+    def callable_action(self, args, kwargs):
         self.double.invoked_method_name = self.attr_name
         return self.double._on_method_call(self, *args, 
-                                                  **kwargs)
-    
-    
+                                                  **kwargs)                
+
+class _HardcodedMethodHandler_(_MethodHandler_):
+    def __init__(self, ret_val):
+        self.ret_val = ret_val
+
+    def callable_action(self, args, kwargs):
+        return self.ret_val                
+
+class _MirrorMethodHandler_(_MethodHandler_):
+    def callable_action(self, args, kwargs):
+        return args[0]           
+
+class _RaiserMethodHandler_(_MethodHandler_):
+    def __init__(self, exception):
+        self.exception = exception
+
+    def callable_action(self, args, kwargs):
+        raise self.exception         
+
 class _Introspector_():
     """
     Internal framework class. Specialized in introspection (reflection)
     """    
-    
     def method_name(self, interceptor):
         return interceptor.attr_name
     
@@ -91,7 +111,6 @@ class _IOParams_():
         return "args=%s,kwargs=%s,output=%s" % (str(self.args),
                                       str(self.kwargs), str(self.output))
     
-        
 class _MethodPool_(object):
     """
     Internal framework class. Intended to store methods info:
@@ -100,7 +119,7 @@ class _MethodPool_(object):
     """
     def __init__(self):
         self.pool = {}
-        self.matchers = _Matchers_()
+        self.match_finder = _MatchFinder_()
     
     def _are_different_arguments(self, arg1, arg2):
         return arg1 != arg2 and \
@@ -112,9 +131,7 @@ class _MethodPool_(object):
             defined_arg = args[i]
             received_arg = last_call_args[i]
             if self._are_different_arguments(defined_arg, received_arg):
-               if defined_arg is None or received_arg is None:
-                   return False
-               if self.matchers.no_one_applies(defined_arg, received_arg):
+               if self.match_finder.args_dont_match(defined_arg, received_arg):
                   return False
         return True
     
@@ -181,8 +198,6 @@ class _MethodPool_(object):
     def stubbed_method_names(self):
         return self.pool.keys()
     
-    
-
 class _PoolReport_():
     """
     Displays the information stored in a MethodPool
@@ -201,8 +216,6 @@ class _PoolReport_():
             return "No one"
         return info
     
-
-
 class _StubPool_(_MethodPool_):
     def _ioparams_for_unspecified_method_input(self, method_name):
         for ioparams in self.pool[method_name]:
@@ -238,14 +251,11 @@ class _StubPool_(_MethodPool_):
         ioparams.kwargs = input_kwargs
         self.pool[method_name][-1] = ioparams
 
-
-        
 class _CallsRepository_():
     """
     Internal framework class. Basically a wrapper around
     the MethodPool, to store calls to methods
     """    
-    
     def __init__(self):
         self.method_pool = _MethodPool_()
         self.report = _PoolReport_(self.method_pool)
@@ -263,8 +273,10 @@ class _CallsRepository_():
             return "No keyword args where passed in"
         return kwargs_str
       
-    def _format_err_msg(self, method_name, args_str, kwargs_str):
-        return "RECORDED calls were: << %s >>, \n EXPECTED call is << args = %s, keyword args = %s >>" % (
+    def _format_err_msg(self, method_name, args, kwargs):
+        args_str = ",".join([str(safeunicode.get_string(arg)) for arg in args])
+        kwargs_str = ",".join([str(safeunicode.get_string(kwarg)) for kwarg in kwargs])
+        return "RECORDED calls were: << %s >>, \n EXPECTED call is << (args = %s), (keyword args = %s) >>" % (
                 str(self.report.method_info(method_name)), 
                 args_str, self._readable_kwargs(kwargs_str))
 
@@ -272,20 +284,16 @@ class _CallsRepository_():
         if not self.method_pool.do_call_args_match(
                         args, kwargs, method_name):
                 raise ArgsDontMatch(self._format_err_msg(
-                         method_name, str(args), str(kwargs)))
+                         method_name, args, kwargs))
                 
     def show_registered_calls(self):
         return self.report.all_stored_methods()
         
-        
-                  
-                  
 class _StubsRepository_():
     """
     Internal framework class. Pretty much a wrapper
     around the StubPool class, to store stubs
     """
-    
     UNDEFINED = "undefined_____"
     
     def __init__(self):
@@ -312,13 +320,13 @@ class _StubsRepository_():
            output = self.method_pool.get_output_for_unspecified_method_input(method_name)
         else:
            output = self.method_pool.get_output_for_specified_method_input(args, kwargs, method_name)
-        if callable(output):
+        if isinstance(output, _MethodHandler_):
            return self._execute_output_method(output, method_name, args, kwargs)
         return output
 
     def _execute_output_method(self, method, name, args, kwargs):
         try:
-           return method(args, kwargs)
+           return method(*args, **kwargs)
         except IndexError:
            raise ApiMismatch(
                 "Method %s seems to require arguments but haven't been passed in" % (
@@ -364,8 +372,6 @@ class _StubsRepository_():
     def stubbed_method_names(self):
         return self.method_pool.stubbed_method_names()
                 
-                
-                
 class _EmptyObject_():
     """
     Internal framework class. Intended to be used as the original_object
@@ -377,5 +383,4 @@ class _EmptyObject_():
         pass
     
     def __getattr__(self, attr):
-        return attr
-                
+        return attr
\ No newline at end of file
