@@ -1,8 +1,3 @@
-from core import _CallsRepository_, _StubsRepository_, _StubPool_
-from core import _EmptyObject_, _Introspector_, _MethodHandler_
-from core import UnexpectedBehavior, WrongApiUsage, ApiMismatch
-from core import ArgsDontMatch, ANY_ARG
- 
 """
 Authors: www.iExpertos.com
    - Carlos Ble (www.carlosble.com)
@@ -11,8 +6,13 @@ License: Apache 2 (http://www.apache.org/licenses/LICENSE-2.0.html)
 
 Project home: http://www.pydoubles.org
 """
-                
-                
+
+from core import _CallsRepository_, _StubsRepository_
+from core import _EmptyObject_, _Introspector_
+from core import _DoubleMethodHandler_, _MirrorMethodHandler_, _RaiserMethodHandler_, _HardcodedMethodHandler_
+from core import UnexpectedBehavior, WrongApiUsage, ApiMismatch
+from core import ArgsDontMatch, ANY_ARG
+
 class ProxySpy(object):
     """
     This test double is just an _interceptor to the original object.
@@ -31,7 +31,7 @@ class ProxySpy(object):
         self.invoked_method_name = None
 
     def __getattr__(self, attr):
-        return _MethodHandler_(self, attr)
+        return _DoubleMethodHandler_(self, attr)
         
     def _before_method(self, args, kwargs):
         self._replace_possible_alias_method_name()
@@ -102,12 +102,10 @@ class ProxySpy(object):
         return self
 
     def then_raise(self, exception_instance):
-        method = method_raising(exception_instance)
-        self._set_output_for_stub(method)
+        self._set_output_for_stub(method_raising(exception_instance))
 
     def then_return_input(self):
-        method = lambda *args, **kwargs: args[0][0]
-        self._set_output_for_stub(method)
+        self._set_output_for_stub(_MirrorMethodHandler_())
         
     def _replace_possible_alias_method_name(self):
         for method_name in self.stubs.stubbed_method_names():
@@ -116,7 +114,6 @@ class ProxySpy(object):
                self.invoked_method_name = method_name
                return
         
-
 class Spy(ProxySpy):
     """
     Works like a ProxySpy but methods are stubbed returning
@@ -158,8 +155,6 @@ class Spy(ProxySpy):
     def _invoke_method(self, args, kwargs):
         return self.DEFAULT_RETURN_VALUE_FOR_STUBBED_METHOD
 
-    
-    
 class Mock(Spy):
     """
     Any method that is going to be invoked on this object must
@@ -200,9 +195,7 @@ class Mock(Spy):
                         self.calls.show_registered_calls(),
                         self.stubs.show_all_methods()))
 
-       
-# PUBLIC STATIC METHODS:        
-
+# PUBLIC STATIC METHODS:
 def proxy_spy(obj_instance):
     """
     Creates a spy objet that records the calls made but passes them
@@ -251,15 +244,13 @@ def method_returning(return_value):
     Creates a method stub, able to receive anything and return 
     the given return_value
     """
-    return lambda *args, **kwargs: return_value
+    return _HardcodedMethodHandler_(return_value)
 
 def method_raising(exception_instance):
     """
     Creates a method stub, which raises the given exception
     """
-    def raise_on_call(*args, **kwargs):
-        raise exception_instance
-    return raise_on_call    
+    return _RaiserMethodHandler_(exception_instance)    
     
 def expect_call(method):
     """
@@ -290,13 +281,11 @@ def assert_that_was_called(method):
         raise WrongApiUsage(
             "Make sure you call assert, passing in a method from a test double: (double.method)")
         
-def assert_that(method):
+def assert_that_method(method):
     """
     Alternative to assert_that_was_called:
     assert_that(obj.method).was_called() 
     """
     double = _Introspector_().double_instance_from_method(method) 
     double.asserting_on_method = method
-    return double
-    
-    
+    return double
\ No newline at end of file
