@@ -1,17 +1,19 @@
 # -*- coding: utf-8 -*-
 
+"""
+Authors:
+    Carlos Ble (www.carlosble.com)
+    Ruben Bernardez (www.rubenbp.com)
+    www.iExpertos.com
+License: Apache 2 (http://www.apache.org/licenses/LICENSE-2.0.html)
+Project home: https://bitbucket.org/carlosble/pydoubles
+"""
+
 import unittest
 from pyDoubles.framework import *
 from pyDoubles.matchers import *
 from pyDoubles.core import _Introspector_
-from pyDoubles.core import _CallsRepository_
-from pyDoubles.core import _IOParams_, _MethodPool_
-
-"""
-Author: Carlos Ble (www.carlosble.com)
-License: Apache 2 (http://www.apache.org/licenses/LICENSE-2.0.html)
-Project home: https://bitbucket.org/carlosble/pydoubles
-"""
+from pyDoubles.core import _MethodPool_
 
 class SomeException(Exception):
     pass
@@ -48,7 +50,6 @@ class Collaborator():
 
 
 class ProxySpyTests(unittest.TestCase):
-    
     def setUp(self):
         self.spy = proxy_spy(Collaborator())
             
@@ -60,7 +61,7 @@ class ProxySpyTests(unittest.TestCase):
     def test_other_way_of_assert_called(self):
         self.spy.hello()
         
-        assert_that(self.spy.hello).was_called()
+        assert_that_method(self.spy.hello).was_called()
 
     def test_assert_was_called_on_any_method(self):
         self.spy.something()
@@ -283,19 +284,19 @@ class ProxySpyTests(unittest.TestCase):
                                ).with_args(1)
                                
     def test_was_never_called(self):
-        assert_that(self.spy.one_arg_method).was_never_called()
+        assert_that_method(self.spy.one_arg_method).was_never_called()
 
     def test_was_never_called_is_false(self):
         self.spy.one_arg_method(1)
         try:
-             assert_that(self.spy.one_arg_method).was_never_called()
+             assert_that_method(self.spy.one_arg_method).was_never_called()
              self.fail("it was called indeed!")
         except UnexpectedBehavior:
             pass
                         
-    
+
 class MethodPoolTests(unittest.TestCase):
-    
+
     def setUp(self):
         self.pool = _MethodPool_()
         self.method_name = "some_method"
@@ -310,12 +311,12 @@ class MethodPoolTests(unittest.TestCase):
         self.assertTrue(
             self.pool.do_call_args_match(
                 (1, ANY_ARG), {}, self.method_name))    
-                
-                
+
+
 class SpyTests(unittest.TestCase):
     def setUp(self):
         self.spy = spy(Collaborator())
-        
+
         
     def test_override_original_method(self):
         self.assertTrue(self.spy.hello() is None)
@@ -373,10 +374,23 @@ class SpyTests(unittest.TestCase):
     def test_assert_was_called_with_method_not_in_the_api(self):
         self.failUnlessRaises(ApiMismatch,
             assert_that_was_called, self.spy.unexisting_method)
-       
+
+
+    def test_do_not_call_callable_object_if_wasnt_generated_by_the_framework(self):
+        class CallableObj():
+            just_testing = True
+            def __call__(self, *args, **kwargs):
+                raise Exception('should not happen')
+            
+        obj = CallableObj()
+        when(self.spy.one_arg_method).then_return(obj)
         
+        self.assertEquals(obj, self.spy.one_arg_method(1),
+                        "Wrong returned object")
+
+
 class MockTests(unittest.TestCase):
-    
+
     def setUp(self):
         self.mock = mock(Collaborator())
         
@@ -518,8 +532,8 @@ class MockTests(unittest.TestCase):
         self.assertEquals(1000, self.mock.one_arg_method(10))
         
         self.mock.assert_that_is_satisfied()
-        
-        
+
+
 class MockFromEmptyObjectTests(unittest.TestCase):
     def setUp(self):
         self.mock = empty_mock()
@@ -655,33 +669,54 @@ class MatchersTests(unittest.TestCase):
         self.spy.one_arg_method(obj)
         
         try:
-            assert_that(self.spy.one_arg_method).was_called().with_args(obj2)
+            assert_that_method(self.spy.one_arg_method).was_called().with_args(obj2)
             self.fail('they should not match')
         except ArgsDontMatch:
             pass    
+        
+    def test_if_doesnt_match_message_is_human_redable(self):
+        self.spy.one_arg_method("XabcX")
+        
+        try:
+            assert_that_was_called(self.spy.one_arg_method).with_args(
+                                    str_containing("xxx"))
+            
+        except ArgsDontMatch, e:
+            self.assertTrue("xxx" in str(e.args), str(e.args))
+            self.assertTrue("string containing" in str(e.args))
+        
                                 
+class CustomMatchersTest(unittest.TestCase):
 
+    def setUp(self):
+        self.spy = spy(Collaborator())
+
+    def test_use_custom_matcher(self):
+        class CustomMatcher(PyDoublesMatcher):
+            matcher_name = "test matcher"
+            def __init__(self, arg):
+                self.defined_arg = arg
+            def matches(self, item):
+                return True
+            
+        when(self.spy.one_arg_method).with_args(
+            CustomMatcher('zzz')).then_return(1000)
+        self.assertEquals(1000, self.spy.one_arg_method('xx'))
+        
+    def test_custom_matcher_do_not_follow_convention(self):
+        class CustomMatcher(PyDoublesMatcher):
+            def matches(self, item):
+                return False
+        
+        self.spy.one_arg_method(1)
+        try:
+            assert_that_was_called(self.spy.one_arg_method).with_args(
+                                   CustomMatcher())
+            self.fail('args dont match!')
+        except ArgsDontMatch:
+            pass        
+        
+        
     
 if __name__ == "__main__":
-    print "Use nosetest to run this tests: nosetest unit.py"
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+    print "Use nosetest to run this tests: nosetest unit.py"
\ No newline at end of file
